@@ -37,7 +37,7 @@
 #include "config.h"
 
 #include <cstring>
-#include <strings.h>
+#include <string.h>
 #include <rak/functional.h>
 #include <rak/string_manip.h>
 
