@@ -86,8 +86,6 @@ Handshake::Handshake(SocketFd fd, HandshakeManager* m, int encryptionOptions) :
   m_uploadThrottle(manager->upload_throttle()->throttle_list()),
   m_downloadThrottle(manager->download_throttle()->throttle_list()),
 
-  m_initializedTime(cachedTime),
-
   m_readDone(false),
   m_writeDone(false),
 
@@ -467,8 +465,8 @@ Handshake::read_info() {
   // Check the first byte as early as possible so we can
   // disconnect non-BT connections if they send less than 20 bytes.
   if ((m_readBuffer.remaining() >= 1 && m_readBuffer.peek_8() != 19) ||
-      m_readBuffer.remaining() >= 20 &&
-      (std::memcmp(m_readBuffer.position() + 1, m_protocol, 19) != 0))
+      (m_readBuffer.remaining() >= 20 &&
+       (std::memcmp(m_readBuffer.position() + 1, m_protocol, 19) != 0)))
     throw handshake_error(ConnectionManager::handshake_failed, e_handshake_not_bittorrent);
 
   if (m_readBuffer.remaining() < part1_size)
@@ -524,6 +522,13 @@ Handshake::read_peer() {
   if (m_peerInfo->supports_extensions())
     write_extension_handshake();
 
+  // Replay HAVE messages we receive after starting to send the bitfield.
+  // This avoids replaying HAVEs for pieces received between starting the
+  // handshake and now (e.g. when connecting takes longer). Ideally we
+  // should make a snapshot of the bitfield here in case it changes while
+  // we're sending it (if it can't be sent in one write() call).
+  m_initializedTime = cachedTime;
+
   // The download is just starting so we're not sending any
   // bitfield. Pretend we wrote it already.
   if (m_download->file_list()->bitfield()->is_all_unset() || m_download->initial_seeding() != NULL) {
@@ -921,7 +926,7 @@ Handshake::prepare_proxy_connect() {
   int advance = snprintf((char*)m_writeBuffer.position(), m_writeBuffer.reserved_left(),
                          "CONNECT %s:%hu HTTP/1.0\r\n\r\n", buf, m_address.port());
 
-  if (advance == -1)
+  if (advance == -1 || advance > m_writeBuffer.reserved_left())
     throw internal_error("Handshake::prepare_proxy_connect() snprintf failed.");
 
   m_writeBuffer.move_end(advance);
@@ -1011,6 +1016,9 @@ Handshake::prepare_peer_info() {
     if (m_peerInfo == NULL)
       throw handshake_error(ConnectionManager::handshake_failed, e_handshake_network_error);
 
+    if (m_peerInfo->failed_counter() > 10)
+      throw handshake_error(ConnectionManager::handshake_dropped, e_handshake_unwanted_connection);
+
     m_peerInfo->set_flags(PeerInfo::flag_handshake);
   }
 
