@@ -1,5 +1,5 @@
 // libTorrent - BitTorrent library
-// Copyright (C) 2005-2006, Jari Sundell
+// Copyright (C) 2005-2007, Jari Sundell
 //
 // This program is free software; you can redistribute it and/or modify
 // it under the terms of the GNU General Public License as published by
@@ -80,6 +80,10 @@ RequestList::delegate() {
 }
 
 // Replace m_canceled with m_queued and set them to stalled.
+//
+// This doesn't seem entirely correct... Perhaps canceled requests
+// should be kept around until we hit a safe state where we may throw
+// them out?
 void
 RequestList::cancel() {
   std::for_each(m_canceled.begin(), m_canceled.end(), std::ptr_fun(&Block::release));
@@ -263,17 +267,17 @@ RequestList::calculate_pipe_size(uint32_t rate) {
   // Change into KB.
   rate /= 1024;
 
-  // Request enough to have enough for 16 seconds at fast rates.
-
   if (!m_delegator->get_aggressive()) {
-    uint32_t queueKB = std::min(rate * 30, (uint32_t)16 << 10);
-  
-    return (queueKB / (Delegator::block_size / 1024)) + 2;
+    if (rate < 20)
+      return rate + 2;
+    else
+      return rate / 5 + 18;
 
   } else {
-    uint32_t queueKB = std::min(rate * 16, (uint32_t)16 << 10);
-  
-    return (queueKB / (Delegator::block_size / 1024)) + 1;
+    if (rate < 10)
+      return rate / 5 + 1;
+    else
+      return rate / 10 + 2;
   }
 }
 
