@@ -105,6 +105,17 @@ FileList::is_valid_piece(const Piece& piece) const {
     piece.offset() + piece.length() <= chunk_index_size(piece.index());
 }
 
+bool
+FileList::is_root_dir_created() const {
+  rak::file_stat fs;
+
+  if (!fs.update(m_rootDir))
+//     return rak::error_number::current() == rak::error_number::e_access;
+    return false;
+
+  return fs.is_directory();
+}
+
 bool
 FileList::is_multi_file() const {
   // Currently only check if we got just one file. In the future this
@@ -454,6 +465,7 @@ FileList::open(int flags) {
   }
 
   m_isOpen = true;
+  m_frozenRootDir = m_rootDir;
 }
 
 void
