@@ -50,6 +50,7 @@
 #ifdef USE_KQUEUE
 #include <sys/types.h>
 #include <sys/event.h>
+#include <sys/select.h>
 #include <sys/time.h>
 #endif
 
@@ -59,31 +60,48 @@ namespace torrent {
 
 inline uint32_t
 PollKQueue::event_mask(Event* e) {
-  return m_table[e->file_descriptor()];
+  assert(e->file_descriptor() != -1);
+
+  Table::value_type entry = m_table[e->file_descriptor()];
+  return entry.second != e ? 0 : entry.first;
 }
 
 inline void
 PollKQueue::set_event_mask(Event* e, uint32_t m) {
-  m_table[e->file_descriptor()] = m;
+  assert(e->file_descriptor() != -1);
+
+  m_table[e->file_descriptor()] = Table::value_type(m, e);
+}
+
+void
+PollKQueue::flush_events() {
+  timespec timeout = { 0, 0 };
+
+  int nfds = kevent(m_fd, m_changes, m_changedEvents, m_events + m_waitingEvents, m_maxEvents - m_waitingEvents, &timeout);
+  if (nfds == -1)
+    throw internal_error("PollKQueue::flush_events() error: " + std::string(rak::error_number::current().c_str()));
+
+  m_changedEvents = 0;
+  m_waitingEvents += nfds;
 }
 
 void
 PollKQueue::modify(Event* event, unsigned short op, short mask) {
-  // Just so I can test it on my mac, comment this out.
-//   if (event->file_descriptor() == 0)
-//     return;
+  // Flush the changed filters to the kernel if the buffer is full.
+  if (m_changedEvents == m_table.size())
+    flush_events();
 
-  // Flush the changed filters to the kernel if the buffer if full.
   if (m_changedEvents == m_maxEvents) {
     if (kevent(m_fd, m_changes, m_changedEvents, NULL, 0, NULL) == -1)
       throw internal_error("PollKQueue::modify() error: " + std::string(rak::error_number::current().c_str()));
-      
+
     m_changedEvents = 0;
   }
 
   struct kevent* itr = m_changes + (m_changedEvents++);
 
-  EV_SET(itr, event->file_descriptor(), mask, op, 0, 0, event);
+  assert(event == m_table[event->file_descriptor()].second);
+  EV_SET(itr, event->file_descriptor(), mask, op, 0, 0, NULL);
 }
 
 PollKQueue*
@@ -100,7 +118,8 @@ PollKQueue::PollKQueue(int fd, int maxEvents, int maxOpenSockets) :
   m_fd(fd),
   m_maxEvents(maxEvents),
   m_waitingEvents(0),
-  m_changedEvents(0) {
+  m_changedEvents(0),
+  m_stdinEvent(NULL) {
 
   m_events = new struct kevent[m_maxEvents];
   m_changes = new struct kevent[maxOpenSockets];
@@ -118,9 +137,24 @@ PollKQueue::~PollKQueue() {
 
 int
 PollKQueue::poll(int msec) {
+#if KQUEUE_SOCKET_ONLY
+  if (m_stdinEvent != NULL) {
+    // Flush all changes to the kqueue poll before we start select
+    // polling, so that they get included.
+    if (m_changedEvents != 0)
+      flush_events();
+
+    if (poll_select(msec) == -1)
+      return -1;
+
+    // The timeout was already handled in select().
+    msec = 0;
+  }
+#endif
+
   timespec timeout = { msec / 1000, (msec % 1000) * 1000000 };
 
-  int nfds = kevent(m_fd, m_changes, m_changedEvents, m_events, m_maxEvents, &timeout);
+  int nfds = kevent(m_fd, m_changes, m_changedEvents, m_events + m_waitingEvents, m_maxEvents - m_waitingEvents, &timeout);
 
   // Clear the changed events even on fail as we might have received a
   // signal or similar, and the changed events have already been
@@ -133,24 +167,67 @@ PollKQueue::poll(int msec) {
   if (nfds == -1)
     return -1;
 
-  m_waitingEvents = nfds;
+  m_waitingEvents += nfds;
 
   return nfds;
 }
 
+#if KQUEUE_SOCKET_ONLY
+int
+PollKQueue::poll_select(int msec) {
+  if (m_waitingEvents >= m_maxEvents)
+    return 0;
+
+  timeval selectTimeout = { msec / 1000, (msec % 1000) * 1000 };
+
+  // If m_fd isn't the first FD opened by the client and has
+  // a low number, using ::poll() here would perform better.
+  //
+  // This kinda assumes fd_set's internal type is int.
+  int readBuffer[m_fd + 1];
+  fd_set* readSet = (fd_set*)&readBuffer;
+
+  std::memset(readBuffer, 0, m_fd + 1);
+  FD_SET(0,    readSet);
+  FD_SET(m_fd, readSet);
+
+  int nfds = select(m_fd + 1, readSet, NULL, NULL, &selectTimeout);
+
+  if (nfds == -1)
+    return nfds;
+
+  if (FD_ISSET(0, readSet)) {
+    m_events[m_waitingEvents].ident = 0;
+    m_events[m_waitingEvents].filter = EVFILT_READ;
+    m_events[m_waitingEvents].flags = 0;
+    m_waitingEvents++;
+  }
+
+  return nfds;
+}
+#endif
+
 void
 PollKQueue::perform() {
   for (struct kevent *itr = m_events, *last = m_events + m_waitingEvents; itr != last; ++itr) {
-    if (itr->flags == EV_ERROR)
-      throw internal_error("PollKQueue::perform() error: " + std::string(rak::error_number(itr->data).c_str()));
+    if (itr->ident < 0 || itr->ident >= m_table.size())
+      continue;
+
+    Table::iterator evItr = m_table.begin() + itr->ident;
+
+    if ((itr->flags & EV_ERROR) && evItr->second != NULL) {
+      if (evItr->first & flag_error)
+        evItr->second->event_error();
+      continue;
+    }
 
     // Also check current mask.
 
-    if (itr->filter == EVFILT_READ && itr->udata != NULL && event_mask((Event*)itr->udata) & flag_read)
-      ((Event*)itr->udata)->event_read();
+    if (itr->filter == EVFILT_READ && evItr->second != NULL && evItr->first & flag_read)
+      evItr->second->event_read();
 
-    if (itr->filter == EVFILT_WRITE && itr->udata != NULL && event_mask((Event*)itr->udata) & flag_write)
-      ((Event*)itr->udata)->event_write();
+    if (itr->filter == EVFILT_WRITE && evItr->second != NULL && evItr->first & flag_write)
+      evItr->second->event_write();
   }
 
   m_waitingEvents = 0;
@@ -169,14 +246,51 @@ PollKQueue::open(Event* event) {
 
 void
 PollKQueue::close(Event* event) {
+#if KQUEUE_SOCKET_ONLY
+  if (event->file_descriptor() == 0) {
+    m_stdinEvent = NULL;
+    return;
+  }
+#endif
+
   if (event_mask(event) != 0)
     throw internal_error("PollKQueue::close(...) called but the file descriptor is active");
 
+  m_table[event->file_descriptor()] = Table::value_type();
+
+  /*
+  Shouldn't be needed anymore.
   for (struct kevent *itr = m_events, *last = m_events + m_waitingEvents; itr != last; ++itr)
     if (itr->udata == event)
       itr->udata = NULL;
 
   m_changedEvents = std::remove_if(m_changes, m_changes + m_changedEvents, rak::equal(event, rak::mem_ref(&kevent::udata))) - m_changes;
+  */
+}
+
+void
+PollKQueue::closed(Event* event) {
+#if KQUEUE_SOCKET_ONLY
+  if (event->file_descriptor() == 0) {
+    m_stdinEvent = NULL;
+    return;
+  }
+#endif
+
+  // Kernel removes closed FDs automatically, so just clear the mask
+  // and remove it from pending calls.  Don't touch if the FD was
+  // re-used before we received the close notification.
+  if (m_table[event->file_descriptor()].second == event)
+    m_table[event->file_descriptor()] = Table::value_type();
+
+  /*
+  for (struct kevent *itr = m_events, *last = m_events + m_waitingEvents; itr != last; ++itr) {
+    if (itr->udata == event)
+      itr->udata = NULL;
+  }
+
+  m_changedEvents = std::remove_if(m_changes, m_changes + m_changedEvents, rak::equal(event, rak::mem_ref(&kevent::udata))) - m_changes;
+  */
 }
 
 // Use custom defines for EPOLL* to make the below code compile with
@@ -203,6 +317,13 @@ PollKQueue::insert_read(Event* event) {
 
   set_event_mask(event, event_mask(event) | flag_read);
 
+#if KQUEUE_SOCKET_ONLY
+  if (event->file_descriptor() == 0) {
+    m_stdinEvent = event;
+    return;
+  }
+#endif
+
   modify(event, EV_ADD, EVFILT_READ);
 }
 
@@ -212,7 +333,6 @@ PollKQueue::insert_write(Event* event) {
     return;
 
   set_event_mask(event, event_mask(event) | flag_write);
-
   modify(event, EV_ADD, EVFILT_WRITE);
 }
 
@@ -227,6 +347,13 @@ PollKQueue::remove_read(Event* event) {
 
   set_event_mask(event, event_mask(event) & ~flag_read);
 
+#if KQUEUE_SOCKET_ONLY
+  if (event->file_descriptor() == 0) {
+    m_stdinEvent = NULL;
+    return;
+  }
+#endif
+
   modify(event, EV_DELETE, EVFILT_READ);
 }
 
@@ -236,7 +363,6 @@ PollKQueue::remove_write(Event* event) {
     return;
 
   set_event_mask(event, event_mask(event) & ~flag_write);
-
   modify(event, EV_DELETE, EVFILT_WRITE);
 }
 
@@ -277,6 +403,10 @@ void
 PollKQueue::close(__UNUSED torrent::Event* event) {
 }
 
+void
+PollKQueue::closed(__UNUSED torrent::Event* event) {
+}
+
 bool
 PollKQueue::in_read(__UNUSED torrent::Event* event) {
   throw internal_error("An PollKQueue function was called, but it is disabled.");
